#include <stdio.h>

/* 
-------------------------------------------------------------------------------
Metabolisme de la vaca
-------------------------------------------------------------------------------
Dades: dt (mida del pas en dies), imax (nombre de passos a calcular),
       jmax (nombre d'iteracions per estabilitzar), qp (activitat de
       l'herba).
Resultats: Al fitxer "vaca.dat". Cada fila representa un instant de temps.
           La primera columna es el temps transcorregut en dies. La seguent
           es l'activitat en Bq a l'interior del tub digestiu. La tercera
           es la contaminacio a la carn en Bq/Kg. La quarta es l'activitat 
           al fetge en Bq i la cinquena la concentracio a la llet en Bq/l.
Dades de prova: dt=0.05 imax=140 jmax=2 qp=5000 ==>
                0.05 423.635 0.00130375 0.0170625 0.0189 3.30019
                0.1 821.271 0.00505146 0.0661097 0.073242 3.30019
                0.15 1194.5 0.0110923 0.145167 0.160851 3.30019
                0.2 1544.83 0.0192845 0.252381 0.279685 3.30019
                ..  ......   .......   ......   ......   .......
Observacions: Des del punt de vista teoric tan sols jmax=1 esta justificat.
              Llavors el metode d'Euler es converteix en un Runge-Kutta.
              Les unitats venen fixades per les variables del programa que
              indiquen la massa de la vaca i el seu volum de llet.
-------------------------------------------------------------------------------
*/

main()
{

int i,j,k,l,imax,jmax;
float dx[4],x0[4],x[4],xf[4],xm[4],font[4],mat[4][4],dt,t0,t,tf,tm,cont,
	k66,k67,k68,k69,k77,k88,lambda,vo,cm,qf,fd,qp,in;
FILE *arxiu;

printf("Mida del pas en dies: ");
scanf("%f",&dt);
printf("Passos que vols calcular: ");
scanf("%d",&imax);
printf("Iteracions per estabilitzar: ");
scanf("%d",&jmax);
printf("Activitat de l'herba en Bq/Kg: ");
scanf("%f",&qp);

/* Calculem la entrada. Una vaca menja uns 5 Kg diaris en pes sec */

qf=5.0;

/* Ara introduim la digestibilitat del cesi: 0.07 per l'herba i 0.2 per
la palla */

fd=0.07;

/* I ja es pot calcular l'entrada */

in=5.0*qf*fd*qp;

/* Comencem a obrir fitxers */

arxiu=fopen("vaca.dat","w");

/* Suposem que inicialment no hi ha contaminacio */

for(k=0;k<4;k++)x0[k]=0.0;
t0=0;

/* Inicialitzem MAT i FONT ... */

for(k=0;k<4;k++)for(l=0;l<4;l++)mat[k][l]=0.0;
for(k=0;k<4;k++)font[k]=0.0;

/* ... i les omplim amb parametres empirics */

lambda=6.33e-5;
k66=1.12e0;
k67=5.96e-2;
k68=1.56e-3;
k69=8.64e-2;
k77=6.91e-3;
k88=6.91e-3;

mat[0][0]=-k66-k67-k68-k69-lambda;
font[0]=in;
mat[1][0]=k67;
mat[1][1]=-k77-lambda;
mat[2][0]=k68;
mat[2][2]=-k88-lambda;
mat[3][0]=k69;
mat[3][3]=-lambda;

vo=50.0;
cm=500.0;

/* Ara inicialitzem les matrius */

for(i=0;i<4;i++)
	{
	x[i]=x0[i];
	xf[i]=x0[i];
	}
t=t0;
for(i=0;i<imax;i++)   
   {
   tm=t+dt/2.0;
   for(j=0;j<jmax;j++)
      {
      for(k=0;k<4;k++)xm[k]=(x[k]+xf[k])/2.0;
      for(k=0;k<4;k++)
         {		
         cont=0.0;
         for(l=0;l<4;l++)cont=cont+mat[k][l]*xm[l];
         cont=cont+font[k];
         dx[k]=cont*dt;
         xf[k]=x[k]+dx[k];
         }         
      }
   for(k=0;k<4;k++)x[k]=xf[k];
   t=t+dt;         

/* I ho treiem per fitxer */

   fprintf(arxiu,"%g %g %g %g %g %g\n",t,x[0],x[1]/cm,x[2],x[3]/vo);

/* Tanquem el bucle... */

   }

/* ... i l'arxiu. */

fclose(arxiu);

/* Final del programa */

}
